#! /usr/bin/python
# -*- coding: iso-8859-15 -*-
#
#     file     : projects/fantec/dtv_converter.py
#     author   : Marcus Rickert (marcus.rickert@web.de)
#     RCS      : $Id$
#     purpose  : script to create a new dtv_channel.txt from an existing one
#     version  : 1.0
#
# calling syntax:
#
#     dtv_converter.py [--build] <DTV-CHANNEL-FILE>
#
# to do:
#   - handle character set correctly

import string
import struct
import binascii
import argparse

from t_AppException import *
from t_OptionHandler import *
from l_crc32 import *
from UserString import MutableString

OPTION_BUILD = "build"

DEFAULT_SOURCE = "dtv_channel.txt"
DEFAULT_TEXT_CONFIG_EXTENSION = ".cfg"
DEFAULT_NEW_DATA_EXTENSION = ".new"
DEFAULT_BUILD = 0

HEADER_SIZE = 8
FOOTER_SIZE = 4
#ENTRY_SIZE = 576
ENTRY_SIZE = 864
CHANNEL_OFFSET = 0
NAME_OFFSET = 8
NAME_LOC_OFFSET = 84
NAME_SIZE = 32
COLUMN_SEPERATOR = "|"
ACTIVE_CHAR = 'a'
INACTIVE_CHAR = 'i'

# -----------------------------------------------------------------------------------

def evaluate(p_in_filename, p_config_filename, p_codepage):

    v_f_in = open(p_in_filename, 'r')
    configData = v_f_in.read()
    v_f_in.close()
    
    version = ord(configData[0])
    if version == 5:
        ENTRY_SIZE = 576
    elif version == 6:
        ENTRY_SIZE = 864
        
    v_f_config = open(p_config_filename, "w")
    pos = HEADER_SIZE
    count = 0
    
    while pos < len(configData) - FOOTER_SIZE:

        idx = ord(configData[pos+CHANNEL_OFFSET])
        lcn  = ord(configData[pos+CHANNEL_OFFSET+2])        
        name = string.strip(configData[pos+NAME_OFFSET:pos+NAME_OFFSET+NAME_SIZE], '\0')

        if ord(configData[pos+NAME_LOC_OFFSET]) == 1:
            name_dtv = string.strip(configData[pos+NAME_LOC_OFFSET+1:pos+NAME_LOC_OFFSET+NAME_SIZE], '\0')
        else:
            name_dtv = string.strip(configData[pos+NAME_LOC_OFFSET:pos+NAME_LOC_OFFSET+NAME_SIZE], '\0')
        
        if not p_codepage is None:
            name = name.decode('utf8').encode(p_codepage).decode('iso-8859-5').encode('utf8')
        
        name_dtv = name_dtv.decode('iso-8859-5').encode('utf-8')
        
        # print channel,'(',lcn, ")", name_dtv
        print lcn, name_dtv
            
        v_f_config.write("%s%s%d%s%s%s%d%s%s\n" % (ACTIVE_CHAR,COLUMN_SEPERATOR,count,COLUMN_SEPERATOR,idx,COLUMN_SEPERATOR,lcn,COLUMN_SEPERATOR, name))
        pos = pos + ENTRY_SIZE
        count = count + 1

    v_f_config.close()
    print "%d channel configuration entries written to %s" % (count, p_config_filename)

# -----------------------------------------------------------------------------------

def build(p_in_filename, p_config_filename, p_out_filename):

    data = MutableString()
    v_f_in = open(p_in_filename, 'r')    
    configData = v_f_in.read()
    v_f_in.close()
    
    version = ord(configData[0])
    if version == 5:
        ENTRY_SIZE = 576
    elif version == 6:
        ENTRY_SIZE = 864
        
    v_f_config = open(p_config_filename, "r")
    v_f_out = open(p_out_filename, "w")
    pos = HEADER_SIZE
    count = 0
    
    for line in v_f_config:

        line = line.rstrip('\n')
        (statusChar, locationString, channelString, numberString, name) = string.split(line, COLUMN_SEPERATOR)

        if statusChar == ACTIVE_CHAR:
            # TODO check channel and number generation
            offset = int(locationString)
            idx = int(channelString)
            lcn = int(numberString)
            entry = configData[HEADER_SIZE + (offset)* ENTRY_SIZE:HEADER_SIZE + (offset+1)*ENTRY_SIZE]
            count = count + 1
            
            # print count, offset, idx, lcn
            
            # set the index, the length of the name and the name itself
            entry = "%s%s%s%s%s%s" % (struct.pack("H", count), struct.pack("H", lcn), struct.pack("i", len(name)), name, chr(0), entry[2*4 + len(name) + 1:])
            data += entry

    v_f_config.close()
    newConfigData = "%s%s%s" % (configData[0:4], struct.pack("i", count), data)  
    v_f_out.write(newConfigData)
    v_f_out.write(struct.pack("I", GetCRC32(newConfigData)))
    v_f_out.close()
    print "%d channel entries written to %s" % (count, p_out_filename)


# -----------------------------------------------------------------------------------

if __name__ == '__main__':

    try:
        '''
        v_Options = [(OPTION_BUILD , "builds the new channel file (instead of evaluating the old one)" )]
        v_OptionHandler = t_OptionHandler(v_Options, 1, 1)
        v_OptionHandler.parse()
        v_DataFilename = v_OptionHandler.f_Args[0]
        v_Build = v_OptionHandler.getParameter(OPTION_BUILD, 0)
        v_ConfigFilename = "%s%s" % (v_DataFilename, DEFAULT_TEXT_CONFIG_EXTENSION)
        '''
        
        parser = argparse.ArgumentParser()
        parser.add_argument('-cp', '--codepage')
        parser.add_argument('--build', dest='build', action='store_true')
        parser.add_argument('filename')
        args = parser.parse_args()
        
        v_DataFilename = args.filename
        v_Build = args.build
        v_Codepage = args.codepage
        
        v_ConfigFilename = "%s%s" % (v_DataFilename, DEFAULT_TEXT_CONFIG_EXTENSION)
        # print v_Codepage

        if v_Build:

            v_newDataFilename = "%s%s" % (v_DataFilename, DEFAULT_NEW_DATA_EXTENSION)
            build(v_DataFilename, v_ConfigFilename, v_newDataFilename)
            
        else:
            
            evaluate(v_DataFilename, v_ConfigFilename, v_Codepage)        
                                    
    except IOError:
        printExceptionMessages()
            
    except t_EnvException:
        printExceptionMessages()
        v_OptionHandler.listOptions();

    except t_AppException:
        printExceptionMessages()
            
    except Exception:
        raise

