# -*- coding: iso-8859-15 -*-
#     file     : python/t_OptionHandler.py
#     author   : Marcus Rickert
#     RCS      : $Id: t_OptionHandler.py,v 1.5 2010-01-24 20:21:29 mr Exp $
#     purpose  : class t_OptionHandler.py
#
#     Copyright (C) 2005 :  Marcus Rickert

# TODO:
#

import getopt
import sys
import string

from t_AppException import *

class t_OptionHandler:

    def __init__(self, p_Options, p_MaxArgs = 0, p_MinArgs = 0):
        
        self.f_Options = [ ]
        self.f_OptionMap = { }
        self.f_MinArgs = p_MinArgs
        self.f_MaxArgs = p_MaxArgs
        for (v_Option, v_Comment) in p_Options:
            v_Parts = string.split(v_Option, "=")
            if len(v_Parts) > 1:
                self.f_Options.append(v_Parts[0] + "=")
                self.f_OptionMap[v_Parts[0]] = [v_Parts[0], "=" + v_Parts[1], v_Comment]
            else:
                self.f_Options.append(v_Parts[0])
                self.f_OptionMap[v_Parts[0]] = [v_Parts[0], "", v_Comment]
                
    # -----------------------------------------------------------------------------------
    
    def parse(self):
        try:
            v_Options, self.f_Args = getopt.getopt(sys.argv[1:], '', self.f_Options)
            if len(self.f_Args) < self.f_MinArgs or len(self.f_Args) > self.f_MaxArgs:
                if self.f_MinArgs == self.f_MaxArgs:
                    raise getopt.error, "%d non-option argument(s) expected, found %d" % \
                          (self.f_MinArgs, len(self.f_Args))
                else:
                    raise getopt.error, "%d..%d non-option argument(s) expected, found %d" % \
                          (self.f_MinArgs, self.f_MaxArgs, len(self.f_Args))
            for (v_Option, v_Value) in v_Options:
                if v_Value == '':
                    v_Value = True
                self.f_OptionMap[v_Option[2:]].append(v_Value)
                
        except getopt.error, v_Cause:
            raise t_EnvException(p_Message="invalid option: %s" % v_Cause)
        
    # -----------------------------------------------------------------------------------
    
    def listOptions(self):

        print "Available options:"
        for (v_Option, v_Parameter, v_Comment) in self.f_OptionMap.values():
            print "--%s%s: %s" % (v_Option, v_Parameter, v_Comment)

    # -----------------------------------------------------------------------------------

    def getOptionKey(self, p_Option):
        
        v_Pos = string.find(p_Option, "=")
        if v_Pos >= 0:
            return p_Option[:v_Pos]
        else:
            return p_Option
        
    # -----------------------------------------------------------------------------------
    
    def isDefined(self, p_Option):
        return len(self.f_OptionMap[self.getOptionKey(p_Option)]) > 3
    
    # -----------------------------------------------------------------------------------
    
    def getParameter(self, p_Option, p_Default=None):
        v_OptionKey = self.getOptionKey(p_Option)
        if self.isDefined(p_Option):
            return self.f_OptionMap[v_OptionKey][3]
        else:
            if p_Default == None:
                raise t_EnvException(p_Message="option '%s' required" % p_Option)
            else:
                return p_Default

    # -----------------------------------------------------------------------------------
    
    def getInteger(self, p_Option, p_Default=None):
        v_OptionKey = self.getOptionKey(p_Option)
        if self.isDefined(p_Option):
            try:
                v_Integer = int(self.f_OptionMap[v_OptionKey][3])

            except Exception:
                raise t_EnvException(p_Message="option '%s' has invalid integer value '%s'" %
                                     (p_Option, self.f_OptionMap[v_OptionKey][3]))

            return v_Integer
        
        else:
            if p_Default == None:
                raise t_EnvException(p_Message="option '%s' required" % p_Option)
            else:
                return p_Default
